/*
 *  server_ip is the IP address of the ESP8266 module, will be 
 *  printed to Serial when the module is connected.
 */

#include <ESP8266WiFi.h>

const char* ssid = "Z21_****";
const char* password = "********";

// Create an instance of the server
// specify the port to listen on as an argument 80
WiFiServer server(4711);

boolean alreadyConnected = false; // whether or not the client was connected previously

// Параметры декодера для настройки:
#define DECODER_ADDRESS_1  22    // Адрес декодера 1
#define DECODER_ADDRESS_2  23    // Адрес декодера 2

// Управляющие пины, где wire номер = номеру провода на схеме светофора Viessmann H0 Licht-Ausfahrsignal Art.Nr  4013:
#define WIRE1_PIN  4
#define WIRE2_PIN  5
#define WIRE3_PIN  14
#define WIRE4_PIN  12
//#define WIRE5_PIN  4

// Флаги  состояний декодера:
bool flag1 = false; // флаг комманды 1 стрелки (ВКЛ/ВЫКЛ)
bool flag2 = false; // флаг комманды 2 стрелки (ВКЛ/ВЫКЛ)
bool flag3 = false; // флаг комманды 3 стрелки (ВКЛ/ВЫКЛ)
bool flag4 = false; // флаг комманды 4 стрелки (ВКЛ/ВЫКЛ)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Функция установки диодов во ВКЛ/ВЫКЛ
void set_lights() {
  // Логика кобработки флагов:
  if ((flag1 == false) && (flag2 == false)) { //Логика состояния 0&0
    // Меняем HIGH на LOW по вкусу! HIGH = диод не горит, LOW = диод горит
    digitalWrite(WIRE1_PIN, LOW); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
  }
  if ((flag1 == false) && (flag2 == true)) { //Логика состояния 0&1
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, LOW); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
  }
  if ((flag1 == true) && (flag2 == false)) { //Логика состояния 1&0
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, LOW); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
  }
  if ((flag1 == true) && (flag2 == true)) { //Логика состояния 1&1
    digitalWrite(WIRE1_PIN, HIGH); digitalWrite(WIRE2_PIN, HIGH); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, LOW);// digitalWrite(WIRE5_PIN, HIGH);
  }
}

// Обработчик стрелочных пакетов. Эта функция запускается, когда библиотека ловит пакет:
void switchSend(byte address, byte data)
{
// Serial.print("Z , ");
// Serial.print(address);
// Serial.print(", ");
// Serial.println(data);
// Обрабатываем адрес стрелки:
 if (address == DECODER_ADDRESS_1) {// Если адрес в пакете == адресу декодера, то обрабатываем.
  switch (data) {
   case 0x00: 
    flag1 = false;
    break;
   case 0x01:
    flag1 = true;
    break;
   default:   // UNKNOWN_COMMAND 
    // Serial.print("UNKNOWN_COMMAND 0x"); 
    // Serial.println(data, HEX);
    break;
  }
 }

 if (address == DECODER_ADDRESS_2) {// Если адрес в пакете == адресу декодера, то обрабатываем.
  switch (data) {    
   case 0x00:
    flag2 = false;
    set_lights(); // меняем схему диодов при переключении
    break;
   case 0x01:
    flag2 = true;
    set_lights(); // меняем схему диодов при переключении
    break;
   default:   // UNKNOWN_COMMAND 
    // Serial.print("UNKNOWN_COMMAND 0x"); 
    // Serial.println(data, HEX);
    break;
  }
 }
}// end of handler

void setup() {
  Serial.begin(115200);
  delay(10);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  //Выставляем управляющие пины на вывод:
  pinMode(WIRE1_PIN, OUTPUT); pinMode(WIRE2_PIN, OUTPUT); pinMode(WIRE3_PIN, OUTPUT); pinMode(WIRE4_PIN, OUTPUT);// pinMode(WIRE5_PIN, OUTPUT);

  //Выставляем их в HIGН, что бы при запуске светофор не горел. Так как они подключены к общему плюсу и минусом к выводам, то управляющая логика будет обратной:
  // HIGH = OFF, LOW = ON:
  digitalWrite(WIRE1_PIN, LOW); digitalWrite(WIRE2_PIN, LOW); digitalWrite(WIRE3_PIN, HIGH); digitalWrite(WIRE4_PIN, HIGH);// digitalWrite(WIRE5_PIN, HIGH);
   
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
  
  Serial.println("new client");  // Wait until the client sends some data
  
  while(!client.available()){
    delay(1);
  }

//--- chtenie stroki
String req = client.readStringUntil('/');

//  Serial.println(req);

//--- obrabotka
byte address = ((req[29]-48)*10)+(req[30]-48);
byte data = req[48] - 48;
switchSend( address, data);
} //END loop
   
//End of sketch

